from datetime import datetime

from NEMO.models import Chemical
from django.forms import (
    BooleanField,
    CharField,
    ModelChoiceField,
    ModelForm,
    ModelMultipleChoiceField,
    Textarea,
    DateField,
    Form,
)
from django.utils import timezone

from NEMO_user_chemicals.models import ChemicalHazard, ChemicalLocation, ChemicalRequest, UserChemical


class ChemicalRequestApprovalForm(ModelForm):

    class Meta:
        model = ChemicalRequest
        fields = ["approved", "approval_comments"]

    def __init__(self, user, *args, **kwargs):
        super(ChemicalRequestApprovalForm, self).__init__(*args, **kwargs)
        self.user = user

    def save(self, commit=True):
        instance = super(ChemicalRequestApprovalForm, self).save(commit=False)
        instance.approver = self.user
        return super(ChemicalRequestApprovalForm, self).save(commit=commit)


class ChemicalRequestForm(ModelForm):

    hazards = ModelMultipleChoiceField(queryset=ChemicalHazard.objects.all(), required=False, label="hazards")

    class Meta:
        model = ChemicalRequest
        fields = [
            "name",
            "cas",
            "container",
            "stability",
            "incompatibilities",
            "hazards",
            "exposure_routes",
            "exposure_controls",
            "procedure",
            "hazardous_waste",
            "waste_disposal",
            "sds",
        ]

    def __init__(self, user, *args, **kwargs):
        super(ChemicalRequestForm, self).__init__(*args, **kwargs)
        self.user = user

    def save(self, commit=True):
        # 1. Create instance but don't save to DB yet (so M2M isn't saved yet either)
        instance = super(ChemicalRequestForm, self).save(commit=False)

        if not instance.pk:
            instance.requester = self.user

        if commit:
            # 3. Save the instance to the DB to generate an ID
            instance.save()
            # 4. Manually save the Many-to-Many data (hazards)
            self.save_m2m()

        return instance


class ChemicalForm(ModelForm):

    hazards = ModelMultipleChoiceField(queryset=ChemicalHazard.objects.all(), required=False, label="hazards")

    class Meta:
        model = Chemical
        fields = [
            "name",
            "keywords",
            "hazards",
            "document",
            "url",
        ]


class UserChemicalForm(ModelForm):

    # Explicitly define the location field to ensure the queryset is correctly populated
    location = ModelChoiceField(queryset=ChemicalLocation.objects.all(), required=True, label="Location")
    comments = CharField(required=False, widget=Textarea(attrs={"rows": 3, "class": "form-control"}), label="Comments")

    class Meta:
        model = UserChemical
        # Added "request" to fields so it can be saved from the hidden input
        fields = ["owner", "label_id", "in_date", "expiration", "chemical", "location", "request"]

    def clean_in_date(self):
        in_date = self.cleaned_data["in_date"]
        return timezone.make_aware(
            datetime(year=in_date.year, month=in_date.month, day=in_date.day), timezone.get_current_timezone()
        )

    def clean_expiration(self):
        expiration = self.cleaned_data["expiration"]
        return timezone.make_aware(
            datetime(year=expiration.year, month=expiration.month, day=expiration.day), timezone.get_current_timezone()
        )


class ChemicalUpdateRequestForm(Form):
    new_owner = CharField(required=False, label="New Owner")
    new_location = CharField(required=False, label="New Location")
    new_expiration = DateField(required=False, label="New Expiration Date")
    new_bottle = BooleanField(required=False, label="Bring in New Bottle")
    remove = BooleanField(required=False, label="Remove/Dispose Chemical")
    other_comments = CharField(
        required=False, widget=Textarea(attrs={"rows": 3, "class": "form-control"}), label="Details"
    )



# Generated by Django 4.2.23 on 2026-01-05 16:18

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("NEMO_user_chemicals", "0006_userchemical_trade_name"),
    ]

    operations = [
        migrations.RenameField(
            model_name="chemicalrequest",
            old_name="chemical_name",
            new_name="name",
        ),
        migrations.RemoveField(
            model_name="chemicalrequest",
            name="trade_name",
        ),
        migrations.RemoveField(
            model_name="userchemical",
            name="trade_name",
        ),
    ]



# Generated by Django 4.2.23 on 2025-09-26 13:02

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("NEMO_user_chemicals", "0003_remove_chemicalrequest_health_hazards"),
    ]

    operations = [
        migrations.AddField(
            model_name="userchemical",
            name="trade_name",
            field=models.CharField(
                blank=True, help_text="Manufacturer/labeled name for this material", max_length=255, null=True
            ),
        ),
    ]



from django.contrib import admin
from django.contrib.admin import register

from NEMO_user_chemicals.models import ChemicalLocation, ChemicalRequest, UserChemical


@register(UserChemical)
class UserChemicalAdmin(admin.ModelAdmin):
    list_display = ("owner", "label_id", "chemical", "expiration")
    autocomplete_fields = ["owner"]


@register(ChemicalRequest)
class ChemicalRequestAdmin(admin.ModelAdmin):
    list_display = ("id", "requester", "date", "name", "approved")
    filter_horizontal = ["hazards"]


@register(ChemicalLocation)
class ChemicalLocationAdmin(admin.ModelAdmin):
    list_display = ("id", "name")



from __future__ import annotations

from NEMO.constants import (
    CHAR_FIELD_LARGE_LENGTH,
    CHAR_FIELD_MEDIUM_LENGTH,
    CHAR_FIELD_SMALL_LENGTH,
)
from NEMO.models import BaseModel, Chemical, ChemicalHazard, User
from NEMO.utilities import get_chemical_document_filename, update_media_file_on_model_update
from django.db import models
from django.dispatch import receiver


class ChemicalRequest(BaseModel):
    class Approval(object):
        PENDING = 0
        APPROVED = 1
        DENIED = 2
        Choices = ((PENDING, "Pending"), (APPROVED, "Approved"), (DENIED, "Denied"))

    requester = models.ForeignKey(
        User, blank=True, null=True, related_name="chemical_requester", on_delete=models.SET_NULL
    )
    approver = models.ForeignKey(
        User, blank=True, null=True, related_name="chemical_approver", on_delete=models.SET_NULL
    )
    date = models.DateTimeField(auto_now_add=True)
    name = models.CharField(max_length=CHAR_FIELD_MEDIUM_LENGTH)
    cas = models.CharField(null=True, blank=True, max_length=CHAR_FIELD_SMALL_LENGTH)
    container = models.CharField(max_length=CHAR_FIELD_SMALL_LENGTH)
    sds = models.FileField(null=True, blank=True, upload_to=get_chemical_document_filename, max_length=500)
    hazards = models.ManyToManyField(ChemicalHazard, blank=True, help_text="Select the hazards for this chemical.")
    stability = models.CharField(max_length=CHAR_FIELD_LARGE_LENGTH)
    incompatibilities = models.CharField(max_length=CHAR_FIELD_LARGE_LENGTH)
    exposure_routes = models.CharField(max_length=CHAR_FIELD_LARGE_LENGTH)
    exposure_controls = models.CharField(max_length=CHAR_FIELD_LARGE_LENGTH)
    procedure = models.TextField()
    hazardous_waste = models.BooleanField()
    waste_disposal = models.TextField()
    approved = models.IntegerField(choices=Approval.Choices, default=Approval.PENDING)
    approval_comments = models.TextField(blank=True, null=True)

    class Meta:
        ordering = ["-date"]
        verbose_name_plural = "Chemical Requests"

    def __str__(self):
        return str(self.id)


@receiver(models.signals.post_delete, sender=ChemicalRequest)
def auto_delete_file_on_chemical_delete(sender, instance: ChemicalRequest, **kwargs):
    """Deletes file from filesystem when corresponding `Chemical` object is deleted."""
    if instance.sds:
        instance.sds.delete(False)


@receiver(models.signals.pre_save, sender=ChemicalRequest)
def auto_update_file_on_chemical_change(sender, instance: ChemicalRequest, **kwargs):
    """Updates old file from filesystem when corresponding `Chemical` object is updated with new file."""
    return update_media_file_on_model_update(instance, "sds")


class ChemicalLocation(BaseModel):
    name = models.CharField(
        max_length=CHAR_FIELD_MEDIUM_LENGTH, help_text="The name for this chemical storage location"
    )

    class Meta:
        verbose_name_plural = "Chemical Storage Location"
        ordering = ["name"]

    def __str__(self):
        return str(self.name)


class UserChemical(BaseModel):
    owner = models.ForeignKey(User, on_delete=models.PROTECT)
    label_id = models.PositiveIntegerField(unique=True)
    chemical = models.ForeignKey(Chemical, blank=True, null=True, on_delete=models.PROTECT)
    request = models.ForeignKey(ChemicalRequest, blank=True, null=True, on_delete=models.SET_NULL)
    in_date = models.DateField()
    expiration = models.DateField()
    location = models.ForeignKey(ChemicalLocation, blank=True, null=True, on_delete=models.SET_NULL)
    history = models.TextField(default="", blank=True)

    class Meta:
        ordering = ["-expiration"]

    def __str__(self):
        return str(self.chemical.name) if self.chemical else "User Chemical"



from django.urls import path, re_path

from NEMO_user_chemicals import views

urlpatterns = [
    # Chemical Requests and Inventory
    path("chemical_dashboard/", views.dashboard, name="user_chemicals_dashboard"),
    path("chemical_request/", views.chemical_request, name="chemical_request"),
    path("chemical_request/edit/<int:request_id>/", views.chemical_request, name="edit_chemical_request"),
    re_path(
        r"^chemical_request/view/(?P<sort_by>requester|date|approved|name)/$",
        views.view_requests,
        name="view_requests",
    ),
    path("chemical_request/view/", views.view_requests, name="view_requests"),
    path("chemical_request/details/<int:request_id>/", views.request_details, name="request_details"),
    path("chemical_request/approval/<int:request_id>/", views.update_request, name="update_request"),
    re_path(
        r"^user_chemicals/(?P<sort_by>owner|chemical|in_date|expiration|location|label_id)/$",
        views.user_chemicals,
        name="user_chemicals",
    ),
    path("user_chemicals/", views.user_chemicals, name="user_chemicals"),
    re_path(
        r"^my_chemicals/(?P<sort_by>owner|chemical|in_date|expiration|location|label_id)/$",
        views.my_chemicals,
        name="my_chemicals",
    ),
    path("my_chemicals/", views.my_chemicals, name="my_chemicals"),
    path("user_chemicals/", views.user_chemicals, name="user_chemicals"),
    path("user_chemicals/add", views.add_user_chemical, name="add_user_chemical"),
    path("user_chemicals/add/<int:chem_req>/", views.add_user_chemical, name="add_user_chemical"),
    path("user_chemicals/update/<int:chem_id>/", views.update_user_chemical, name="update_user_chemical"),
    path("user_chemicals/request_update/<int:chem_id>/", views.request_chemical_update, name="request_chemical_update"),
    path("user_chemicals/delete/<int:chem_id>/", views.delete_user_chemical, name="delete_user_chemical"),
    path("chemical/add/", views.add_chemical, name="add_chemical"),
    path("chemical/add/<int:request_id>/", views.add_chemical, name="add_chemical_from_request"),
]



# Generated by Django 4.2.23 on 2025-09-04 15:36

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("NEMO_user_chemicals", "0002_chemicallocation_remove_chemicalrequest_corrosive_and_more"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="chemicalrequest",
            name="health_hazards",
        ),
    ]



from datetime import timedelta

from NEMO.models import Chemical, ChemicalHazard, User
from NEMO.utilities import EmailCategory, render_email_template, send_mail
from NEMO.views.customization import get_media_file_contents
from django.contrib.admin.views.decorators import staff_member_required
from django.contrib.auth.decorators import login_required
from django.http import HttpResponseRedirect, HttpResponseForbidden
from django.shortcuts import get_object_or_404, redirect, render
from django.urls import reverse
from django.utils import timezone
from django.views.decorators.http import require_GET, require_POST, require_http_methods

from NEMO_user_chemicals.customizations import ChemicalsCustomization
from NEMO_user_chemicals.forms import (
    ChemicalForm,
    ChemicalRequestApprovalForm,
    ChemicalRequestForm,
    ChemicalUpdateRequestForm,
    UserChemicalForm,
)
from NEMO_user_chemicals.models import ChemicalLocation, ChemicalRequest, UserChemical
from django.utils.safestring import mark_safe


def send_new_chemical_request_email(chemical_request, request, is_edit=False):
    message = get_media_file_contents("chemical_request_email.html")
    chem_request_emails = ChemicalsCustomization.get("chemical_request_email_addresses")
    if message and chem_request_emails:
        dictionary = {
            "request": chemical_request,
            "url": request.build_absolute_uri(reverse("view_requests")),
            "request_edited": is_edit,
        }
        subject_prefix = "Updated" if is_edit else "New"
        subject = f"{subject_prefix} chemical request from {chemical_request.requester.get_full_name()}"
        content = render_email_template(message, dictionary)
        recipients = tuple([e for e in chem_request_emails.split(",") if e])
        send_mail(
            subject=subject,
            content=content,
            to=recipients,
            from_email=chemical_request.requester.email,
            email_category=EmailCategory.SAFETY,
        )


def send_chemical_request_email_update(chemical_request):
    message = get_media_file_contents("chemical_request_update_email.html")
    chem_request_emails = ChemicalsCustomization.get("chemical_request_email_addresses")
    if message and chem_request_emails:
        subject = f"Update to your Material Request for {chemical_request.name}"
        message = render_email_template(message, {"request": chemical_request})
        recipients = tuple([e for e in chem_request_emails.split(",") if e])
        send_mail(
            subject=subject,
            content=message,
            from_email=chemical_request.approver.email,
            to=[chemical_request.requester.email],
            cc=recipients,
            email_category=EmailCategory.SAFETY,
        )


@login_required
@require_GET
def dashboard(request):
    return render(request, "NEMO_user_chemicals/dashboard.html")


@login_required
@require_http_methods(["GET", "POST"])
def chemical_request(request, request_id=None):
    hazards = ChemicalHazard.objects.all()

    # Check if we are editing an existing request
    chem_req = None
    if request_id:
        chem_req = get_object_or_404(ChemicalRequest, id=request_id)
        # Only allow staff or the original requester to edit
        if not request.user.is_staff and chem_req.requester != request.user:
            return HttpResponseForbidden("You do not have permission to edit this request.")

    if request.method == "POST":
        # Pass instance=chem_req to update the existing record
        form = ChemicalRequestForm(request.user, data=request.POST, files=request.FILES, instance=chem_req)
        if form.is_valid():
            req = form.save(commit=False)

            is_edit = False
            # If editing, reset status to Pending so it gets reviewed again
            if request_id:
                req.approved = ChemicalRequest.Approval.PENDING
                is_edit = True

            req.save()
            form.save_m2m()  # Essential when using commit=False

            # Send email with edit flag using original logic
            send_new_chemical_request_email(req, request, is_edit)

            dictionary = {
                "title": "Request received",
                "heading": "Your request has been received and will be reviewed shortly.",
                "content": mark_safe(
                    f"You can view the status of your request on the <a href='{reverse('view_requests')}'>Material Requests</a> page."
                ),
            }
            return render(request, "acknowledgement.html", dictionary)
        else:
            dictionary = {"hazards": hazards, "form": form, "chemical_request": chem_req}
            return render(request, "NEMO_user_chemicals/chemical_request.html", dictionary)

    # GET Request
    dictionary = {"hazards": hazards}
    if chem_req:
        dictionary["chemical_request"] = chem_req
        # Initialize form with instance data
        form = ChemicalRequestForm(request.user, instance=chem_req)
        dictionary["form"] = form
    else:
        dictionary["form"] = ChemicalRequestForm(request.user)

    return render(request, "NEMO_user_chemicals/chemical_request.html", dictionary)


@login_required
@require_GET
def view_requests(request, sort_by="date"):
    if request.user.is_staff:
        pending_requests = ChemicalRequest.objects.filter(approved=ChemicalRequest.Approval.PENDING)
        all_requests = ChemicalRequest.objects.all()
    else:
        pending_requests = ChemicalRequest.objects.filter(
            requester=request.user, approved=ChemicalRequest.Approval.PENDING
        )
        all_requests = ChemicalRequest.objects.filter(requester=request.user)
    if sort_by == "requester":
        pending_requests = pending_requests.order_by("requester__first_name")
        all_requests = all_requests.order_by("requester__first_name")
    elif sort_by == "name":
        pending_requests = pending_requests.order_by("name")
        all_requests = all_requests.order_by("name")
    elif sort_by == "approved":
        all_requests = all_requests.order_by("approved")
    else:
        pending_requests = pending_requests.order_by("-date")
        all_requests = all_requests.order_by("-date")

    dictionary = {"pending_requests": pending_requests, "all_requests": all_requests}
    return render(request, "NEMO_user_chemicals/view_requests.html", dictionary)


@login_required
@require_GET
def request_details(request, request_id):
    req = get_object_or_404(ChemicalRequest, id=request_id)
    if not request.user.is_staff and req.requester != request.user:
        return HttpResponseRedirect(reverse("view_requests"))

    dictionary = {"chemical_request": req, "hazards": req.hazards.all()}

    # Check if this is an AJAX request using header OR explicit GET param
    if request.headers.get("x-requested-with") == "XMLHttpRequest" or request.GET.get("ajax"):
        dictionary["base_template"] = "NEMO_user_chemicals/ajax_base.html"
    else:
        dictionary["base_template"] = "base.html"

    return render(request, "NEMO_user_chemicals/request_details.html", dictionary)


@staff_member_required(login_url=None)
@require_POST
def update_request(request, request_id):
    req = get_object_or_404(ChemicalRequest, id=request_id)
    form = ChemicalRequestApprovalForm(request.user, request.POST, instance=req)
    if form.is_valid():
        form.save()
        send_chemical_request_email_update(req)
        dictionary = {
            "title": "Request Status Updated",
            "heading": "You successfully updated the status of this request",
        }
        return render(request, "acknowledgement.html", dictionary)
    dictionary = {
        "title": "Request Status Update Failed",
        "heading": "There was a problem with this update.",
    }
    return render(request, "acknowledgement.html", dictionary)


@login_required
@require_GET
def user_chemicals(request, sort_by="chemical"):
    if request.user.is_staff:
        user_chems = UserChemical.objects.all()
    else:
        user_chems = UserChemical.objects.filter(owner=request.user)

    if sort_by == "owner":
        user_chems = user_chems.order_by("owner__first_name")
    elif sort_by == "chemical":
        user_chems = user_chems.order_by("chemical__name")
    elif sort_by == "label_id":
        user_chems = user_chems.order_by("label_id")
    elif sort_by == "expiration":
        user_chems = user_chems.order_by("expiration")
    elif sort_by == "location":
        user_chems = user_chems.order_by("location__name")
    else:
        user_chems = user_chems.order_by("-in_date")

    dictionary = {"user_chemicals": user_chems}
    return render(request, "NEMO_user_chemicals/user_chemicals.html", dictionary)


@login_required
@require_GET
def my_chemicals(request, sort_by="chemical"):
    user_chems = UserChemical.objects.filter(owner=request.user)

    if sort_by == "owner":
        user_chems = user_chems.order_by("owner__first_name")
    elif sort_by == "chemical":
        user_chems = user_chems.order_by("chemical__name")
    elif sort_by == "label_id":
        user_chems = user_chems.order_by("label_id")
    elif sort_by == "expiration":
        user_chems = user_chems.order_by("expiration")
    elif sort_by == "location":
        user_chems = user_chems.order_by("location__name")
    else:
        user_chems = user_chems.order_by("-in_date")

    dictionary = {"user_chemicals": user_chems, "view_name": "my_chemicals", "page_title": "My Chemicals"}
    return render(request, "NEMO_user_chemicals/user_chemicals.html", dictionary)


@staff_member_required(login_url=None)
@require_http_methods(["GET", "POST"])
def add_user_chemical(request, chem_req=None):
    if request.method == "GET":
        chem_request = get_object_or_404(ChemicalRequest, id=chem_req) if chem_req else None
        dictionary = {
            "one_year_from_now": timezone.now() + timedelta(days=365),
            "today": timezone.localdate(),
            "users": User.objects.filter(is_active=True),
            "chemicals": Chemical.objects.all(),
            "chemical_request": chem_request,
            "locations": ChemicalLocation.objects.all(),
        }
        return render(request, "NEMO_user_chemicals/add_user_chemical.html", dictionary)
    elif request.method == "POST":
        form = UserChemicalForm(data=request.POST)
        if form.is_valid() and not form.cleaned_data.get("chemical"):
            form.add_error("chemical", "You must select a valid chemical from the list.")
        if not form.is_valid():
            dictionary = {
                "title": "Chemical request failed",
                "heading": "Invalid form data",
                "content": str(form.errors),
            }
            return render(request, "acknowledgement.html", dictionary)

        user_chem = form.save(commit=False)
        # Fetch comments from cleaned_data, or fallback to request.POST
        comments = form.cleaned_data.get("comments") or request.POST.get("comments", "")
        timestamp = timezone.now().strftime("%Y-%m-%d %H:%M")
        history_entry = f"[{timestamp}] Added: Owner: {user_chem.owner.get_full_name()}, Label ID: {user_chem.label_id}, Date In: {user_chem.in_date}, Expiration: {user_chem.expiration}. Comments: {comments}\n"
        user_chem.history = (user_chem.history or "") + history_entry
        user_chem.save()
        return HttpResponseRedirect(reverse("user_chemicals"))


@staff_member_required(login_url=None)
@require_http_methods(["GET", "POST"])
def update_user_chemical(request, chem_id):
    user_chem = get_object_or_404(UserChemical, id=chem_id)
    if request.method == "GET":
        form = UserChemicalForm(instance=user_chem)
        owner = user_chem.owner
        dictionary = {
            "form": form,
            "one_year_from_now": timezone.now() + timedelta(days=365),
            "users": User.objects.filter(is_active=True),
            "owner": owner,
            # Added chemicals to context for typeahead
            "chemicals": Chemical.objects.all(),
            "locations": ChemicalLocation.objects.all(),
        }
        return render(request, "NEMO_user_chemicals/update_user_chemical.html", dictionary)
    elif request.method == "POST":
        user_chem = get_object_or_404(UserChemical, id=chem_id)
        form = UserChemicalForm(data=request.POST, instance=user_chem)
        if form.is_valid() and not form.cleaned_data.get("chemical"):
            form.add_error("chemical", "You must select a valid chemical from the list.")
        if not form.is_valid():
            dictionary = {
                "title": "Chemical request update failed",
                "heading": "Invalid form data",
                "content": str(form.errors),
            }
            return render(request, "acknowledgement.html", dictionary)

        user_chem = form.save(commit=False)
        # Fetch comments from cleaned_data, or fallback to request.POST
        comments = form.cleaned_data.get("comments") or request.POST.get("comments", "")
        timestamp = timezone.now().strftime("%Y-%m-%d %H:%M")
        history_entry = f"[{timestamp}] Updated: Owner: {user_chem.owner.get_full_name()}, Label ID: {user_chem.label_id}, Date In: {user_chem.in_date}, Expiration: {user_chem.expiration}. Comments: {comments}\n"
        user_chem.history = (user_chem.history or "") + history_entry
        user_chem.save()
        return HttpResponseRedirect(reverse("user_chemicals"))


@login_required
@require_http_methods(["GET", "POST"])
def request_chemical_update(request, chem_id):
    user_chem = get_object_or_404(UserChemical, id=chem_id)
    # Only allow the owner or staff to request an update
    if not request.user.is_staff and user_chem.owner != request.user:
        return HttpResponseForbidden("You do not have permission to modify this chemical.")

    if request.method == "POST":
        form = ChemicalUpdateRequestForm(request.POST)
        if form.is_valid():
            # Email logic
            chem_request_emails = ChemicalsCustomization.get("chemical_request_email_addresses")
            if chem_request_emails:
                recipients = tuple([e for e in chem_request_emails.split(",") if e])

                if recipients:
                    details = ""
                    selected_actions_display = []

                    if form.cleaned_data.get("new_owner"):
                        details += f"- New Owner: {form.cleaned_data['new_owner']}\n"
                        selected_actions_display.append("Change Owner")
                    if form.cleaned_data.get("new_location"):
                        details += f"- New Location: {form.cleaned_data['new_location']}\n"
                        selected_actions_display.append("Change Location")
                    if form.cleaned_data.get("new_bottle"):
                        selected_actions_display.append("Bring in New Bottle")
                    if form.cleaned_data.get("new_expiration"):
                        details += f"- New Expiration: {form.cleaned_data['new_expiration']}\n"
                        selected_actions_display.append("Adjust Expiration")
                    if form.cleaned_data.get("remove"):
                        selected_actions_display.append("Remove/Dispose Chemical")
                    if form.cleaned_data.get("other_comments"):
                        details += f"- Other Details: {form.cleaned_data['other_comments']}\n"
                        selected_actions_display.append("Other")

                    # Check if anything was actually requested
                    if not selected_actions_display:
                        form.add_error(None, "Please select at least one action or fill in a field.")
                        return render(
                            request, "NEMO_user_chemicals/request_update.html", {"form": form, "user_chem": user_chem}
                        )

                    subject = f"Chemical Update Request: {user_chem.chemical.name} (ID: {user_chem.label_id})"
                    content = (
                        f"User {request.user.get_full_name()} has requested an update for the following chemical:\n\n"
                        f"Chemical: {user_chem.chemical.name}\n"
                        f"Label ID: {user_chem.label_id}\n"
                        f"Current Location: {user_chem.location}\n\n"
                        f"Requested Actions: {', '.join(selected_actions_display)}\n\n"
                        f"Details:\n{details}"
                    )

                    send_mail(
                        subject=subject,
                        content=content,
                        to=recipients,
                        from_email=request.user.email,
                        email_category=EmailCategory.SAFETY,
                    )

                    dictionary = {
                        "title": "Update Request Sent",
                        "heading": "Your update request has been sent to staff.",
                        "content": mark_safe(
                            f"You will be contacted if further information is needed. Return to <a href='{reverse('my_chemicals')}'>My Chemicals</a>."
                        ),
                    }
                    return render(request, "acknowledgement.html", dictionary)
            dictionary = {
                "title": "Update Request Problem",
                "heading": "There was a problem submitting your update request. Please contact staff",
                "content": mark_safe(f"Return to <a href='{reverse('my_chemicals')}'>My Chemicals</a>."),
            }
            return render(request, "acknowledgement.html", dictionary)
    else:
        form = ChemicalUpdateRequestForm()

    return render(request, "NEMO_user_chemicals/request_update.html", {"form": form, "user_chem": user_chem})


@staff_member_required(login_url=None)
@require_POST
def delete_user_chemical(request, chem_id):
    user_chem = get_object_or_404(UserChemical, id=chem_id)
    user_chem.delete()
    return HttpResponseRedirect(reverse("user_chemicals"))


@staff_member_required(login_url=None)
@require_http_methods(["GET", "POST"])
def add_chemical(request, request_id=None):
    chem_req = None
    if request_id:
        chem_req = get_object_or_404(ChemicalRequest, id=request_id)

    if request.method == "POST":
        form = ChemicalForm(data=request.POST, files=request.FILES)
        if form.is_valid():
            chemical = form.save(commit=False)
            # If no document uploaded but we have a request with an SDS, copy it
            if not chemical.document and chem_req and chem_req.sds:
                chemical.document.save(chem_req.sds.name, chem_req.sds)
            chemical.save()
            form.save_m2m()
            return redirect("user_chemicals_dashboard")
    else:
        initial_data = {}
        if chem_req:
            initial_data["name"] = chem_req.name
            # Use values_list to get IDs so they match the form choices in the template
            initial_data["hazards"] = chem_req.hazards.values_list("id", flat=True)

        form = ChemicalForm(initial=initial_data)

    return render(request, "NEMO_user_chemicals/add_chemical.html", {"form": form, "source_request": chem_req})



from NEMO.decorators import customization
from NEMO.utilities import render_email_template
from NEMO.views.customization import CustomizationBase, get_media_file_contents
from django.core.validators import validate_email


@customization("chemicals", "User Chemicals")
class ChemicalsCustomization(CustomizationBase):
    variables = {"chemical_request_email_addresses": ""}
    files = [("chemical_request_email", ".html"), ("chemical_request_update_email", ".html")]

    def validate(self, name, value):
        if name == "chemical_request_email_addresses":
            recipients = tuple([e for e in value.split(",") if e])
            for email in recipients:
                validate_email(email)

    @staticmethod
    def render_template(template_name, dictionary: dict, request=None):
        template = get_media_file_contents(template_name)
        return render_email_template(template, dictionary, request=request)



from django.apps import AppConfig


class NemoUserChemicalsConfig(AppConfig):
    name = "NEMO_user_chemicals"
    verbose_name = "NEMO User Chemicals"

    def ready(self):
        """
        This code will be run when Django starts.
        """
        pass



from setuptools import find_namespace_packages, setup

VERSION = '2.1.1'

setup(
    name="NEMO-user-chemicals",
    version=VERSION,
    description="User chemical request and inventory management plugin for NEMO",
    packages=find_namespace_packages(),
    author="David Barth",
    author_email="dsbarth@seas.upenn.edu",
    url="https://gitlab.com/nemo-community/upenn/nemo_user_chemicals",
    include_package_data=True,
    install_requires=[
        "django",
    ],
    extras_require={
        "NEMO": ["NEMO>=7.0.0"],
        "dev-tools": ["pre-commit", "djlint", "black"],
    },
    keywords=["NEMO"],
)



# Generated by Django 3.2.19 on 2023-05-30 18:00

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("NEMO", "0042_version_4_3_0"),
    ]

    operations = [
        migrations.CreateModel(
            name="ChemicalRequest",
            fields=[
                ("id", models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("date", models.DateTimeField(auto_now_add=True)),
                ("chemical_name", models.CharField(max_length=200)),
                ("cas", models.CharField(max_length=100)),
                ("container", models.CharField(max_length=100)),
                ("sds_link", models.URLField()),
                ("flammable", models.BooleanField()),
                ("corrosive", models.BooleanField()),
                ("reactive", models.BooleanField()),
                ("temp_sensitive", models.BooleanField()),
                ("stability", models.CharField(max_length=500)),
                ("incompatibilities", models.CharField(max_length=500)),
                ("health_hazards", models.CharField(max_length=500)),
                ("exposure_routes", models.CharField(max_length=500)),
                ("exposure_controls", models.CharField(max_length=500)),
                ("procedure", models.TextField()),
                ("hazardous_waste", models.BooleanField()),
                ("waste_disposal", models.TextField()),
                ("approved", models.IntegerField(choices=[(0, "Pending"), (1, "Approved"), (2, "Denied")], default=0)),
                ("approval_comments", models.TextField(blank=True, null=True)),
                (
                    "approver",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="chemical_approver",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "requester",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="chemical_requester",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "verbose_name_plural": "Chemical Requests",
                "ordering": ["-date"],
            },
        ),
        migrations.CreateModel(
            name="UserChemical",
            fields=[
                ("id", models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("label_id", models.PositiveIntegerField(unique=True)),
                ("chemical_name", models.CharField(max_length=200)),
                ("sds_link", models.URLField(blank=True, null=True)),
                ("in_date", models.DateField()),
                ("expiration", models.DateField()),
                ("location", models.CharField(max_length=100)),
                ("owner", models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL)),
                (
                    "request",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="NEMO_user_chemicals.chemicalrequest",
                    ),
                ),
            ],
            options={
                "ordering": ["-expiration"],
            },
        ),
    ]



# Generated by Django 4.2.23 on 2025-08-29 20:17

import NEMO.utilities
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("NEMO_user_chemicals", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="ChemicalLocation",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("name", models.CharField(help_text="The name for this chemical storage location", max_length=255)),
            ],
            options={
                "verbose_name_plural": "Chemical Storage Location",
                "ordering": ["name"],
            },
        ),
        migrations.RemoveField(
            model_name="chemicalrequest",
            name="corrosive",
        ),
        migrations.RemoveField(
            model_name="chemicalrequest",
            name="flammable",
        ),
        migrations.RemoveField(
            model_name="chemicalrequest",
            name="reactive",
        ),
        migrations.RemoveField(
            model_name="chemicalrequest",
            name="sds_link",
        ),
        migrations.RemoveField(
            model_name="chemicalrequest",
            name="temp_sensitive",
        ),
        migrations.RemoveField(
            model_name="userchemical",
            name="chemical_name",
        ),
        migrations.RemoveField(
            model_name="userchemical",
            name="sds_link",
        ),
        migrations.AddField(
            model_name="chemicalrequest",
            name="hazards",
            field=models.ManyToManyField(
                blank=True, help_text="Select the hazards for this chemical.", to="NEMO.chemicalhazard"
            ),
        ),
        migrations.AddField(
            model_name="chemicalrequest",
            name="sds",
            field=models.FileField(
                blank=True, max_length=500, null=True, upload_to=NEMO.utilities.get_chemical_document_filename
            ),
        ),
        migrations.AddField(
            model_name="userchemical",
            name="chemical",
            field=models.ForeignKey(
                blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to="NEMO.chemical"
            ),
        ),
        migrations.AlterField(
            model_name="chemicalrequest",
            name="chemical_name",
            field=models.CharField(max_length=255),
        ),
        migrations.AlterField(
            model_name="chemicalrequest",
            name="exposure_controls",
            field=models.CharField(max_length=1024),
        ),
        migrations.AlterField(
            model_name="chemicalrequest",
            name="exposure_routes",
            field=models.CharField(max_length=1024),
        ),
        migrations.AlterField(
            model_name="chemicalrequest",
            name="health_hazards",
            field=models.CharField(max_length=1024),
        ),
        migrations.AlterField(
            model_name="chemicalrequest",
            name="id",
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID"),
        ),
        migrations.AlterField(
            model_name="chemicalrequest",
            name="incompatibilities",
            field=models.CharField(max_length=1024),
        ),
        migrations.AlterField(
            model_name="chemicalrequest",
            name="stability",
            field=models.CharField(max_length=1024),
        ),
        migrations.AlterField(
            model_name="userchemical",
            name="id",
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID"),
        ),
        migrations.AlterField(
            model_name="userchemical",
            name="location",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="NEMO_user_chemicals.chemicallocation",
            ),
        ),
    ]



# Generated by Django 4.2.23 on 2026-01-06 15:04

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("NEMO_user_chemicals", "0007_rename_chemical_name_chemicalrequest_name_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="userchemical",
            name="history",
            field=models.TextField(blank=True, default=""),
        ),
    ]



# Generated by Django 4.2.27 on 2026-01-16 16:01

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("NEMO_user_chemicals", "0008_userchemical_history"),
    ]

    operations = [
        migrations.AlterField(
            model_name="chemicalrequest",
            name="cas",
            field=models.CharField(blank=True, max_length=100, null=True),
        ),
    ]
